/*
	 This program is used to test the function that gets u from
	 the slope of data points (alpha) in the slope matching method.
	 It can be used to compare results with Sen (1986) Table 1
	 from Ground Water Vol 24, #2, articles titled "Determination
	 of Aquifer Parameters by the Slope-Matching Method"

		This program accompanies the article
	 'Computer determination of aquifer parameters by slope Matching'
	 by Robert Pittenger and James S. Reichard.
   The program is written by Robert Pittenger

*/


#include <iostream.h>
#include <math.h>



//  calculates factorial of arg by Brute Force
double fac(int n)
{
	switch (n)
	{
		case 1:
		{
			return(1.0);
		}
		case 2:
		{
			return(2.0);
		}
		case 3:
		{
			return(6.0);
		}
		case 4:
		{
			return(24.0);
		}
		case 5:
		{
			return(120.0);
		}
		case 6:
		{
			return(720.0);
		}
		case 7:
		{
			return(5040.0);
		}
		case 8:
		{
			return(40320.0);
		}
		case 9:
		{
			return(362880.0);
		}
		case 10:
		{
			return(3628800.0);
		}
		case 11:
		{
			return(39916800.0);
		}
		case 12:
		{
			return(479001600.0);
		}
		case 13:
		{
			return(6227020800.0);
		}
		default:
			return(1);
	}
}// end of factorial




// calcualtes Theis well function for hydrogeology
double WellFunction(double u)
{
	if (u <= 0.0) return(0.0); // check for errors
	double val;
	val = -0.5772 - log(u) + u - pow(u, 2.0)/(2.0*fac(2)) + pow(u,3.0)/(3.0*fac(3))
		- pow(u,4.0)/(4.0*fac(4)) + pow(u,5.0)/(5.0*fac(5)) - pow(u,6.0)/(6.0*fac(6))
		+ pow(u,7.0)/(7.0*fac(7)) - pow(u,8.0)/(8.0*fac(8))
		+ pow(u,9.0)/(9.0*fac(9)) -	pow(u,10.0)/(10.0*fac(10))
		+ pow(u,11.0)/(11.0*fac(11)) - pow(u,12.0)/(12.0*fac(12));
	return (val);
}

// calcualtes Theis well function for hydrogeology using long doubles
// In C++, it is allowable to give two functions the same name
// if the parameters are different.  The computer selects the function
// to call based on the parameter
long double WellFunction(long double u)
{
	if (u <= 0.0) return(0.0); // check for errors
	long double val;
	val = -0.5772 - logl(u) + u - powl(u, 2.0)/(2.0*fac(2)) + powl(u,3.0)/(3.0*fac(3))
		- powl(u,4.0)/(4.0*fac(4)) + powl(u,5.0)/(5.0*fac(5)) - powl(u,6.0)/(6.0*fac(6))
		+ powl(u,7.0)/(7.0*fac(7)) - powl(u,8.0)/(8.0*fac(8))
		+ powl(u,9.0)/(9.0*fac(9)) -	powl(u,10.0)/(10.0*fac(10))
		+ powl(u,11.0)/(11.0*fac(11)) - powl(u,12.0)/(12.0*fac(12));
	return (val);
}



double GetU(double slope)
//  This function calculates u from slope matching
// it returns a zero if it is out of calcuable range
// u is r*r*S/(4*T*t)
{

	double guess, u, logu;
	int counter = 0;
	double err = .1;
	double lowerbound = -10.0;
	double upperbound = log10(9.0);

	if (slope < 0.0) slope = -slope;
	if (slope > pow(10.0, lowerbound) && slope < pow(10.0, upperbound))
	{
		logu = -5.0;
		u = pow(10.0, logu);
		guess = -exp(-u)/WellFunction(u);
		if (guess < 0) guess = -guess;
		do
		{
			if (guess < slope) // move right
			{
				lowerbound = logu;
				logu = (upperbound+lowerbound)/2.0;
			}
			if (guess > slope) // move left
			{
				upperbound = logu;
				logu = (upperbound+lowerbound)/2.0;
			}
			u = pow(10.0, logu);
			err = fabs(slope - guess);
			guess = -exp(-u)/WellFunction(u);
			if (guess < 0.0) guess = -guess;
			++counter;
		}
		while (err > .00001 && counter < 500);
	}  // end if
	else u = 0.0;
	cout<<"counter is "<<counter<<endl;
	return(u);
} // end of function




main()
{
	cout<<"This program is used to test the function GetU()"<<endl;
	cout<<"used in the slope matching calculations"<<endl;
	cout<<"as part of the article 'Computer determination of"<<endl;
	cout<<"aquifer parameters by slope matching'"<<endl;
  cout<<"by Robert Pittenger and James S Reichard."<<endl<<endl;
	cout<<"Results should be compared with Table 1 from Sen (1986)"<<endl;
	cout<<"'Determination of Aquifer Parameters by Slope Matching' "<<endl;
  cout<<"in 'Ground Water', Mar-Apr, 1989"<<endl<<endl;
	cout<<"To use the program, you enter a slope value from which"<<endl;
	cout<<"u is to be determined."<<endl;
	cout<<"Enter 0.0  to exit"<<endl<<endl;
	double value1, value2;
	do
	{
		cout<<"Enter the value to test (0.0 to exit): ";
		cin>>value1;
		if (value1 == 0.0) return(0);
		value2 = GetU(value1);
		cout<<"For slope of "<<value1<<", u is "<<value2<<endl;
//		value2 = GetKoArg(value2);
//		cout<<"Does "<<value1<<" = "<<value2<<endl<<endl;
	}
	while(value1 != 0.0);
	cout<<"Good-Bye"<<endl;
	return(0);
}