/*
	 This program is used to test the function that gets the arguments
	 to the Bessel function Ko.  It accompanies the article
	 'Computer determination of aquifer parameters by slope Matching'
	 by Robert Pittenger and James S Reichard.
   The program is written by Robert Pittenger

	 The functions bessio and bessko are from:
	 Press, William, H., Saul A. Teukolsky, William T. Vetterling,
	 and Brian P. Flannery. 1992. 'Numerical recipes in C:
	 The art of scientific computing,' Cambridge University Press.

	 and is subject to that sources copyright restrictions

*/


#include <iostream.h>
#include <math.h>


double bessio(double x)
// returns the modified bessel function Io(x) for any real x
// from Press et al., 'Numerical Recipes in C', 1992
{
	double ax, ans, y;

	ax = fabs(x);
	if (ax < 3.75)
	{
		y = x / 3.75;
		y*=y;
		ans = 1.0+y*(3.5156229+y*(3.0899424+y*(1.2067492
			+y*(0.2659732+y*(0.360768e-1+y*0.45813e-2)))));
	}
	else
	{
		y = 3.75/ax;
		ans = (exp(ax)/sqrt(ax))*(0.39894228+y*(0.1328592e-1
			+y*(0.225319e-2+y*(-0.157565e-2+y*(0.916281e-2
			+y*(-0.2057706e-1+y*(0.2635537e-1+y*(-0.1647633e-1
			+y*0.392377e-2))))))));
	}
	return(ans);
}



double bessko(double x)
// returns the modified Bessel function Ko for positive real x
// from Press et al. 'Numerical Recipies in C', 1992
{
	double bessio(double x);
	double y, ans;
	if (x <= 2.0)
	{
		y = x*x/4.0;
		ans = (-log(x/2.0)*bessio(x))+(-0.57721566+y*(0.42278420
			+y*(0.23069756+y*(0.3488590e-1+y*(0.262698e-2
			+y*(0.10750e-3+y*0.74e-5))))));
	}
	else
	{
	y = 2.0/x;
	ans = (exp(-x)/sqrt(x))*(1.25331414+y*(-0.7832358e-1
		+y*(0.2189568e-1+y*(-0.1062446e-1+y*(0.587872e-2
		+y*(-0.251540e-2+y*0.53208e-3))))));
	}
	return(ans);
}  // end of bessko





// this functions returns the value x for a specified y for
// the equation y = Ko(x)  where Ko is a bessel function
float GetKoArg(float check)
{
	float x = 1, max = 1, min = 0, error = 1, result;
	do
	{
		result = bessko(max);
		if (result > check)
			max = max + 1;
	}
	while (result > check);
	do
	{
		result = bessko(x);

		if (result < check) // decrease x
		{
			max = x;
			x = (x + min)/2.0;
		}
		else // increase x
		{
			min = x;
			x = (x + max)/ 2.0;
		}
		error = fabs(result-check);
	}
	while (error > 0.00001);
	return(x);
}

main()
{
	cout<<"This program is used to test the function GetKoArg()"<<endl;
	cout<<"used in Hantush's slope matching calculation"<<endl;
	cout<<"as part of the article 'Computer determination of"<<endl;
	cout<<"aquifer parameters by slope matching'"<<endl;
  cout<<"by Robert Pittenger and James S Reichard"<<endl<<endl;
	cout<<"To use the program, you enter a value to test"<<endl;
	cout<<"The program calulates and displays the bessel function of"<<endl;
	cout<<"the value.  The new value is used to test GetKoArg"<<endl<<endl;
	cout<<"The final value should be the number you first entered"<<endl;
	cout<<"Enter negative value to exit"<<endl<<endl;
	double value1, value2;
	do
	{
		cout<<"Enter the value to test (negative to exit): ";
		cin>>value1;
    if (value1 < 0.0) return(0);
		value2 = bessko(value1);
		cout<<"Ko("<<value1<<") = "<<value2<<endl;
		value2 = GetKoArg(value2);
		cout<<"Does "<<value1<<" = "<<value2<<endl<<endl;
	}
	while(value1 != -1.0);
	cout<<"Good-Bye"<<endl;
	return(0);
}